export { withPageContext };
import type { PageContext } from 'vike/types';
import type { StateCreator, StoreMutatorIdentifier } from 'zustand';
type WithPageContext = <T, Mps extends [StoreMutatorIdentifier, unknown][] = [], Mcs extends [StoreMutatorIdentifier, unknown][] = []>(f: (pageContext: PageContext) => StateCreator<T, Mps, Mcs>) => StateCreator<T, Mps, Mcs>;
/**
 * Middleware to make `pageContext` available to the store.
 *
 * Example usage:
 *
 * ```ts
 *
 * interface Store {
 *   user: {
 *     id: number
 *     firstName: string
 *   }
 * }
 *
 * const useStore = create<Store>()(
 *   withPageContext((pageContext) => (set, get) => ({
 *     user: pageContext.user
 *   }))
 * )
 * ```
 */
declare const withPageContext: WithPageContext;
